/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// created by : r.nagel 4.4.2005
//
// function : central configuration instance of non-graphical application properties
//
// todo     : load/save the properties
//
//
// modified :

package net.sf.langproper ;

import net.sf.langproper.engine.*;
import java.util.logging.*;
import java.io.*;

public class TGlobal
{
  public static final String RESOURCE_ROOT = "/resource" ;
  public static final String LANGUAGES_FILENAME = "iso639.txt" ;
  public static final String COUNTRY_FILENAME = "iso3166.txt" ;
  public static final String LOGGER_PROPS_FILENAME = "logger.properties" ;

  public static String DEFAULT_LOAD_PATH = "./" ;

  public static String APPLICATION_NAME = "Popeye" ;
  public static String VERSION_STRING = "version 0.21" ;

  // the text in language properties files
  public static String INFO_COMMENT_TEXT = "#! created/edited by " ;

  // home of popeye
  public static String PROJECT_MAIN_URL = "popeye.sourceforge.net" ;

  public static TGlobal runtime = new TGlobal() ;

  public static final TLangProps current = new TLangProps() ;


  private boolean replaceSpaces ;

  // --------------------------------------------------------------------------

  private TGlobal()
  {
    LogManager logMan = LogManager.getLogManager() ;

    try
    {
      InputStream is = TGlobal.class.getResourceAsStream(RESOURCE_ROOT + "/" + LOGGER_PROPS_FILENAME ) ;
      if (is != null)
        logMan.readConfiguration(is) ;
      else  // no logging config => turn logging off
      {
        System.out.println( "logger config not readable" ) ;
        Logger.global.setLevel( Level.OFF );
      }
    }
    catch (Exception ex)
    {
      System.out.println( "logger config not readable" ) ;
      ex.printStackTrace();
      Logger.global.setLevel( Level.OFF );
    }

    replaceSpaces = true ;
  }

  // --------------------------------------------------------------------------
  /** if it returns true then saveFile in TLangProps creates a .bak file */
  public boolean makeBackupFiles()
  {
    return true ;
  }

  // --------------------------------------------------------------------------
  /** if it returns true, then keys without a translation were saved into
   *  every translation file (empty value) .
   * Otherwise only keys with a translation were saved into the translation
   * file.
   * e.g. if languages lang1, lang2, lang3 available and there
   * are some definitions like : key=lang1, key=lang3
   * true  => key were saved into lang1, lang2 and lang3 file
   * false => key were saved into lang1 and lang3
   */
  public boolean saveEmptyKeys()
  {
    return true ;
  }


  // --------------------------------------------------------------------------
  /** if it returns true, all whitespaces between 2 words are replaced by "_"  */
  public boolean replaceSpaceInInput()
  {
    return replaceSpaces ;
  }

  public void setReplaceSpaceInput(boolean newValue)
  {
    replaceSpaces = newValue ;
  }

  // --------------------------------------------------------------------------

}
