/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// created by : r.nagel 4.4.2005
//
// function : central data model, holds all data from the files
//            additional data model for jtable
//
// todo     : - it should be redesigned  (put defaultLang into langVersions) !!
//
//
// modified : 2.05.2005
//            hasComment(..) method added

package net.sf.langproper.engine ;

import java.io.* ;
import java.util.* ;
import javax.swing.table.* ;
import net.sf.langproper.*;
import java.util.logging.*;

public class TLangProps
    extends AbstractTableModel

{
  public static final int
      SORT_NOTHING = -1,
      SORT_UPWARD = 8,
      SORT_DOWNWARD = 11,
      SORT_TRANSLATIONSCOUNT_UPWARD = 13,
      SORT_TRANSLATIONSCOUNT_DOWNWARD = 14
      ;

  /** contains all translations */
  private TranslationList translations ;

  /** sorting direction */
  private int sortDirection = SORT_NOTHING ;

  /** data available */
  private boolean dataSetLoaded = false ;

  /** data were changed - every data manipulation operation must set this flag */
  private boolean dataChanged = false ;

  /** true if a comment was found */
  private boolean commentFound = false ;

  /** list of all available languages and some of it's properties  */
  private TLanguageList langList ;

  /** record for table column -> languageID */
  private int[] mapping ;

  // --------------------------------------------------------------------------

  public TLangProps()
  {
    translations = new TranslationList() ;
    langList = new TLanguageList() ;
  }

  // --------------------------------------------------------------------------

  // load all .properties files
  public void load( File file )
  {
    // splitt the filename
    TLanguageVersionFile firstLoadFile = new TLanguageVersionFile( file ) ;

    String suffix = firstLoadFile.getSuffix() ;

    commentFound = false ;

    if ( suffix.toLowerCase().hashCode() == ".properties".hashCode() )
    {
      File dir = new File( file.getParent() ) ;

      File files[] = dir.listFiles() ;
      if ( files != null )
      {
        langList.clear() ;

        // collect all filenames
        for ( int t = 0 ; t < files.length ; t++ )
        {
          TLanguageVersionFile workName = new TLanguageVersionFile( files[t] ) ;

          // load only files with an equal filename (but different language/country extension)
          if ( workName.isEqual( firstLoadFile ) )
          {
            // append language
            langList.addLanguageVersion( workName ) ;
          }
        }

        // arrange the languages files in an alphabetical order
        langList.sortElements();

        // load all files into datamodel
        reload() ;

        dataSetLoaded = true ;
        dataChanged = false ;
      }
      else
      {
        Logger.global.info( " could not found any language bundle file " ) ;
      }
    }
  }

  /** load all files into MM */
  public void reload()
  {
    // delete all data
    translations.clear() ;

    // load default file
    if ( langList.getDefaultLang() != null )
    {
      loadSingleFile( langList.getDefaultLang(), langList.DEFAULT_LANG ) ;
    }
    // load all other language files
    for ( int t = 0 ; t < langList.size() ; t++ )
    {
      loadSingleFile( langList.get( t ), t ) ;
    }
    this.updateData() ;
  }

  // load a single key=value property file
  private boolean loadSingleFile( TLanguageVersionFile file, int langID )
  {
    String buf = null ;
    int trenner = 0 ;
    int zeile = 1 ;
    boolean back = false ;
    try
    {
      String key, value ;

      TComments comment = null ;

      BufferedReader input = new BufferedReader(
          new InputStreamReader(
              new FileInputStream( file.getFile() ) ), 1000 ) ;
      while ( input.ready() )
      {
        buf = input.readLine() ;

        if ( buf != null )
        {
          buf = buf.trim() ; // returns a copy of the string, with leading and trailing whitespace omitted.
          if ( buf.length() > 0 ) // no empty lines
          {
            if ( buf.charAt( 0 ) != '#' ) // no comment line
            {

              trenner = buf.indexOf( '=' ) ;

              if ( ( trenner == -1 ) || ( trenner == 0 ) ) // no =
              {
                Logger.global.info( file.getFile() +" --> \"=\" not found at line : " + zeile ) ;
                Logger.global.info("line : " +buf) ;
              }
              else
              {
                key = buf.substring( 0, trenner ) ;
                value = buf.substring( trenner + 1 ) ;
                addTranslation( key, value, comment, langID ) ;
                comment = null ;
              }
            }
            else  // comment line
            {
              boolean insert = true ;
              // a "#!...." line isn't a real comment => ignore it
              if (buf.length() > 1)
              {
                if (buf.charAt(1) == '!')
                {
                  insert = false ;
                }
              }
              if (insert) // no "#!" comment
              {
                if ( comment == null )
                  comment = new TComments() ;
                comment.addComment( buf ) ;
                commentFound = true ;
              }
            }
          }
        }

        zeile++ ;
      }
      input.close() ;
      back = true ;
    }
    catch ( Exception e )
    {
//      System.out.println( e ) ;
      e.printStackTrace() ;
      Logger.global.warning( " erro loading file " +file.getFileName() ) ;
    }

    return back ;
  }

  // --------------------------------------------------------------------------

  /** save all files */
  public void save()
  {
    if ( langList.getDefaultLang() != null )
    {
      saveSingleFile( langList.getDefaultLang(), langList.DEFAULT_LANG ) ;
    }
    // load all other language files
    for ( int t = 0 ; t < langList.size() ; t++ )
    {
      saveSingleFile( langList.get( t ), t ) ;
    }
    dataChanged = false ;
  }

  private void saveSingleFile( TLanguageVersionFile file, int langID )
  {
    try
    {
      String fileName = file.getFile().getCanonicalPath() ;

      if (TGlobal.runtime.makeBackupFiles())
      {
        File bakFile = new File( fileName ) ;
        if ( !bakFile.renameTo( new File( fileName + ".bak" ) ) )
        {
//          System.out.println( fileName + ".bak could not be created !" ) ;
          Logger.global.warning( fileName + ".bak could not be created !" ) ;
        }
      }
      PrintWriter out = new PrintWriter( new BufferedWriter(
                               new FileWriter( fileName) ) ) ;

      // save the info text (applicationname and version)
      out.println(TGlobal.INFO_COMMENT_TEXT
                 + TGlobal.APPLICATION_NAME
                 + " " +TGlobal.VERSION_STRING
                 + " (" +TGlobal.PROJECT_MAIN_URL +")");
      out.println() ;

      for (Enumeration myEnum = translations.elements() ; myEnum.hasMoreElements() ; )
      {
        TMultiLanguageEntry entry = (TMultiLanguageEntry) myEnum.nextElement() ;

        boolean save = false ;
        String trans = entry.getTranslation(langID) ;

        // save keys with empty values ?
        if (TGlobal.runtime.saveEmptyKeys())
        {
          save = true ;
        }
        else // key with value ?
        {
          if ( trans != null )
            if ( trans.length() > 0 )
            {
              save = true ;
            }
        }

        if (save)
        {
          // handling of comments
          TComments comment = entry.getComments(langID) ;
          if (comment != null)
          {
            int cSize = comment.getSize() ;
            for (int t = 0 ; t < cSize ; t++)
            {
              out.println( comment.getFormatedComment(t));
//              System.out.println( "comment " +comment.getFormatedComment(t) ) ;
            }
          } // all available comments saved

          // save key=value pair
          out.print( entry.getKey() ) ;
          out.print( "=" ) ;

          // prevent a key=null line in saved file
          if ( trans != null )
            out.println( trans ) ;
          out.println() ;
        }

        entry.setStatus( TMultiLanguageEntry.STATE_UNCHANGED);
      }

      out.flush();
      out.close();

    }
    catch ( Exception e )
    {}
  }

  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------


  // --------------------------------------------------------------------------

  public TMultiLanguageEntry addTranslation( String key, String value, int langID )
  {
    return addTranslation(key, value, null, langID) ;
  }

  public TMultiLanguageEntry addTranslation( String key, String value,
                                             TComments comment,
                                             int langID )
  {
//    System.out.println("try #" +key +"#");
    key = Utils.get_Unicode_I18n_String(key) ;
    value = Utils.get_Unicode_I18n_String(value) ;

    TMultiLanguageEntry entry = translations.get( key ) ;

    if ( entry == null )
    {
      entry = new TMultiLanguageEntry( langList.size() ) ;
      entry.setKey( key ) ;
      translations.add( entry ) ;
    }

    entry.setTranslation( value, comment, langID ) ;
    dataChanged = true ;

    return entry ;
  }

  // returns a translation
  // langID = 0    => key
  //        = 1    => default (if available) or first available language translation
  //        = 2..x => other translations
  public String getTranslation( TMultiLanguageEntry entry, int langID )
  {
    String back = null ;
    if ( langID > -1)
    {
      if ( langID == 0 )
      {
        back = entry.getKey() ;
      }
      else
      {
         int id = this.getMappedIndex(langID-1) ;
         back = entry.getTranslationOnly( id ) ;
         if (back == null)
           back = "" ;
      }
    }
    else back = "" ;

    return back ;
  }

  // replace a translation
  // langID = 0    => key
  //        = 1    => default (if available) or first available language translation
  //        = 2..x => other translations
  public void replaceTranslation( TMultiLanguageEntry entry, int langID,
                                  String value )
  {
    if ( entry == null )
    {
      return ;
    }

    if ( langID == 0 )
    {
      entry.setKey( value ) ;
    }
    else
    {
      int id = this.getMappedIndex(langID-1) ;
      entry.setTranslation( value, id ) ;
    }
    dataChanged = true ;
    this.fireTableRowsUpdated(0, this.getSize()-1);
  }

  /** remove a translation entry <entry> from list */
  public void removeTranslation( TMultiLanguageEntry entry )
  {
    if (translations.remove( entry ))
    {
      dataChanged = true ;
      this.fireTableRowsDeleted( 0, this.getSize() - 1 ) ;
    }
  }

  /** remove the translation entry on index <index> from list */
  public void removeTranslation( int index )
  {
    if (translations.remove( index ))
    {
      dataChanged = true ;
      this.fireTableRowsDeleted( index, index ) ;
    }
  }


  /** search the entry with key <key> and returns the TMultiLanguageEntry object */
  public TMultiLanguageEntry getLanguageEntry( String key )
  {
     return translations.get( key ) ;
  }

  /** returns the index of entry with key <key> or -1 if no entry in translationlist */
  public int getLanguageEntryIndex( String key )
  {
     return translations.getIndex( key ) ;
  }

  /** put a new TMultiLanguageEntry with key <key> into the translation list,
   *  the method doesn't check, if the key is already definied
   */
  public TMultiLanguageEntry addLanguageEntry( String key )
  {
    TMultiLanguageEntry entry = new TMultiLanguageEntry( langList.size() ) ;
    entry.setKey( key ) ;
    translations.add( entry ) ;

    dataChanged = true ;
    this.fireTableRowsInserted(translations.size(), translations.size());

    return entry ;
  }

// ----------------------------------------------------------------------------


  /** changes the sortDirection of data list */
  public void setSortDirection(int newSortDirection)
  {
    if (newSortDirection != sortDirection)
    {
      translations.sortElements( newSortDirection, 0);
      sortDirection = newSortDirection ;
    }
  }

  public int getSortDirection()
  {
     return sortDirection ;
  }

  // ----------------------------------------------------------------------------

  /** returns the size */
  public int getSize()
  {
    return translations.size() ;
  }

  /** returns the number of available and visible languages */
  public int getLangSize()
  {
//    return langList.size() ;
      return langList.getVisibleSize() ;
  }

  /** returns true if a default translation is available */
  public boolean hasDefaultEntry()
  {
    if ( langList.getDefaultLang()  == null )
    {
      return false ;
    }

    return true ;
  }

  public TLanguageList getAvailableLangs()
  {
    return langList ;
  }

  /** returns the status of internal data */
  public boolean hasData()
  {
    return dataSetLoaded ;
  }

  public boolean isDataChanged()
  {
    return dataChanged ;
  }

  public boolean isCommentFound()
  {
    return commentFound ;
  }

  /** checks if any translation entry on index <index> has a comment */
  public boolean hasComment(int index)
  {
    TMultiLanguageEntry dummy = translations.get(index) ;
    if (dummy != null)
      return dummy.hasComment() ;  // has any translation a comment

    return false ;
  }

  /** repaint table data */
  public void updateData()
  {
    this.updateMapping() ;
//    this.fireTableDataChanged() ;
    this.fireTableStructureChanged() ;
  }

// ----------------------------------------------------------------------------
//     mapping of table coulmns -> langID
// ----------------------------------------------------------------------------

  // every visible language get its own table column
  private void updateMapping()
  {
    int len = langList.size() ;
    int vLen = langList.getVisibleSize() ; // number of visible language versions
    int off = 0 ;

    if ( langList.getDefaultLang() != null )
    {
      mapping = new int[vLen+1] ;
      mapping[0] = TLanguageList.DEFAULT_LANG ;
      off = 1 ;
    }
    else
      mapping = new int[vLen] ;

    // generate the mapping
    for ( int t = 0 ; t < len ; t++)
    {
      if ( langList.get(t).isVisible())
      {
        mapping[off] = t ;
        off++ ;
      }
    }

  }

  // returns the language-version-objecct for column index
  private TLanguageVersionFile getMapped_LVF_Object(int index)
  {
    TLanguageVersionFile back = null ;

    if (index > -1)
    {
      int col = getMappedIndex(index) ;
      if (col != langList.UNDEFINED_LANG)
      {
        if ( col == langList.DEFAULT_LANG )
        {
          back = langList.getDefaultLang() ;
        } else
        {
          back = langList.get(col) ;
        }
      }
    }

    return back ;
  }

  // returns the language-version-ID (TLanguageVersionFile) for column index
  // 0 = default lang (if available)
  // 1 = first visible lang
  // 2 = ...
  private int getMappedIndex(int index)
  {
    int back = TLanguageList.UNDEFINED_LANG ;

    if ((index >= 0) && (index < mapping.length))
    {
      back = mapping[index] ;
    }
/*
    if (index > -1)
    {
        // no default
        if ( langList.getDefaultLang() != null )
        {
          if ( index == 0 ) // default entry
          {
            back = langList.DEFAULT_LANG ;
          }
          else
          {
            back = index - 1 ;  // calculate language index
          }
        }
        else  // no default entry
        {
          back = index ;
        }
    }
*/
    return back ;
  }


// ----------------------------------------------------------------------------
//     AbstractDataModel
// ----------------------------------------------------------------------------

  /**
   * getColumnCount
   *
   * @return int
   */
  public int getColumnCount()
  {
    int back = getLangSize() + 2 ;  // column and key !
    if ( langList.getDefaultLang()  != null )
    {
      back++ ;
    }
    return back ;
  }

  /*
   * JTable uses this method to determine the default renderer/
   * editor for each cell.  If we didn't implement this method,
   * then the last column would contain text ("true"/"false"),
   * rather than a check box.
   */
  public Class getColumnClass( int c )
  {
    return String.class ;
  }

  public String getColumnName( int col )
  {
    String back = "-" ;
    if ( col == 0 ) // status column
    {
      back = "" ;
    }
    else if ( col == 1 ) // key
    {
      back = "key" ;
    }
    else   // language column
    {
      // is a default entry available ?
      TLanguageVersionFile dummy = this.getMapped_LVF_Object(col-2) ;
      if (dummy != null) // entry available
      {
         back = dummy.getFullLanguageName() ;

         if (col == 2)  // default entry position ?
         {
           if (langList.getDefaultLang() != null)  // default available
           {
             back = "default" ;
             if ( langList.size() < 1 ) // but no other languages
               back = "value" ;
           }
         }
      }
    }
    return back ;
  }

  /**
   * getRowCount
   *
   * @return int
   */
  public int getRowCount()
  {
    return getSize() ;
  }

  /**
   * getValueAt
   *
   * @param rowIndex int
   * @param columnIndex int (not used)
   * @return Object of type TSingleLanguageEntry
   */
  public Object getValueAt( int rowIndex, int columnIndex )
  {
    return translations.get( rowIndex ) ;
  }

}
