/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// created by : r.nagel 4.4.2005
//
// function : name handling of one properties language file
//
// todo     :
//
//
// modified : 09.05.2005
//            insert a "visible" Boolean object, used by TPropTable (hide some languages)

package net.sf.langproper.engine ;

import java.io.*;
import net.sf.langproper.engine.iso.*;

/** splitts an Java Language Properties file (prefix, _language, .suffix) */

public class TLanguageVersionFile
{
  private File fullName ;
  private String prefix ;
  private String suffix ;
  private String lang ;     // 2 letter language ID
  private String country ;  // 2 letter country ID
  private String version ;  // lang and country ID

  private Boolean visible ; // visibility option in LangManagingDialog

  public TLanguageVersionFile()
  {
    prefix = "" ;
    suffix = "" ;
    lang = "" ;
    country = "" ;
    version = "" ;
    visible = new Boolean(true) ;
  }

  public TLanguageVersionFile(File file)
  {
    init(file) ;
  }


  public void init(File file)
  {
    visible = new Boolean(true) ;
    fullName = file ;
    String fileName = fullName.getName() ;

    int i1 = fileName.indexOf( '_' ) ;
    int i2 = fileName.indexOf( '.', -1 ) ;

    prefix = "" ;
    suffix = "" ;

    if (i1 > 0)  // has _ => filename with language extension
    {
      prefix = fileName.substring(0, i1) ;
      if (i2 > 0)
      {
        suffix = fileName.substring(i2) ;
        version = fileName.substring(i1+1, i2) ;
      }
      else  // no valid suffix
      {
        version = fileName.substring(i1+1) ;
      }
    }
    else // no _ => no language extension
    {
      version = "" ;
      if (i2 > 0)  // only . in filename => no language extension
      {
        prefix = fileName.substring( 0, i2 ) ;
        suffix = fileName.substring(i2) ;
      }
      else  // no _ and . => filename without fileextension like .properties
      {
        prefix = fileName ;
      }
    }

    if (version.length() > 0)
    {
      // full structure "_languageID_countryID" = version

      i1 = version.indexOf( '_' ) ;
      if (i1 > 1) // ignore any leading _
      {
        country = version.substring(i1+1) ;
        lang = version.substring(0, i1) ;
      }
      else
        lang = version ;
    }
//    System.out.println("filename #" +fileName +"# lang =" +lang +":") ;
  }

  // check's if the prefix and suffix data are
  public boolean isEqual(TLanguageVersionFile other)
  {
    boolean back = false ;

    if (prefix.hashCode() == other.prefix.hashCode())
    {
      if (suffix.hashCode() == other.suffix.hashCode())
      {
        back = true ;
      }
    }
    return back ;
  }

  public String getPrefix()
  {
    return prefix;
  }

  public String getSuffix()
  {
    return suffix;
  }

  public String getVersion()
  {
    return version;
  }

  /** returns the "english" language name and the available ids
   *  like "German (de_DE)".
   */
  public String getFullLanguageName()
  {
    if (country != null)
      if (country.length() > 0)
        return TLanguageNames.runtime.getLanguageNameExt(lang, country) ;

    return TLanguageNames.runtime.getLanguageNameExt(lang) ;
  }

  /** returns the 2letter iso country code from file name */
  public String getCountryCode()
  {
    return country ;
  }

  /** returns the 2letter iso language code from file name */
  public String getLanguageCode()
  {
    return lang ;
  }

  public boolean hasLanguageExtension()
  {
    boolean back = false ;
    if (lang != null)
      if (lang.length() > 0)
        back = true ;
    return back ;
  }

  public String getFileName()
  {
    return fullName.getName() ;
  }

  public File getFile()
  {
    return fullName ;
  }

  /** returns the boolean Object */
  public Boolean getVisible()
  {
    return visible;
  }

  /** returns a boolean value */
  public boolean isVisible()
  {
    return visible.booleanValue() ;
  }

  public void setVisible(boolean pVisible)
  {
      visible = new Boolean(pVisible) ;
  }

}
