/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// created by : r.nagel 4.4.2005
//
// function : contains a list of all keys, every key has several translations
//            -> list of TMultiLanguageEntry
//
// todo     :
//
// modified :

package net.sf.langproper.engine ;

import java.util.*;

public class TranslationList
{
  private Vector translations ;

  public TranslationList()
  {
    translations = new Vector() ;
  }


  /** returns the first entry with key <key> */
  public TMultiLanguageEntry get( String key )
  {
    if ( key == null)
      return null ;

    TMultiLanguageEntry back = null ;
    int keyHash = key.hashCode() ;

    Enumeration myEnum = translations.elements() ;
    while ( (myEnum.hasMoreElements()) && (back == null) )
    {
      TMultiLanguageEntry dummy = (TMultiLanguageEntry) myEnum.nextElement() ;
      if (dummy.getKey().hashCode() == keyHash)
      {
        back = dummy ;
      }
    }

    return back ;
  }

  /** returns the entry on index <index> */
  public TMultiLanguageEntry get( int index )
  {
    TMultiLanguageEntry back = null ;

    if ( (index >= 0) && (index < translations.size()) )
    {
      back = (TMultiLanguageEntry) translations.get(index) ;
    }

    return back ;
  }

  /** returns the index of entry with key <key> or -1 if no entry in list */
  public int getIndex( String key )
  {
    int back = -1 ;

    if ( key != null)
    {
      if (key.length() > 0)
      {
        int keyHash = key.hashCode() ;
        int counter = 0 ;

        Enumeration myEnum = translations.elements() ;
        while ( ( myEnum.hasMoreElements() ) && ( back < 0 ) )
        {
          TMultiLanguageEntry dummy = ( TMultiLanguageEntry ) myEnum.
              nextElement() ;
          if ( dummy.getKey().hashCode() == keyHash )
          {
            back = counter ;  // found -> save index
          }
          counter++ ;
        }
      }
    }
    return back ;

  }


  /** Adds a new entry into the list.
   *  There is no check whether the element is already contained.
   */
  public void add( TMultiLanguageEntry entry )
  {
     translations.add( entry ) ;
  }


  public boolean remove( TMultiLanguageEntry entry )
  {
    if (entry == null)
      return false ;

    // try to find entry in list and remove it
    return remove( getIndex( entry.getKey() )) ;
  }

  public boolean remove( int index )
  {
    // valid entry index
    if ( (index > -1) && (index < translations.size()) )
    {
      translations.remove( index ) ;
      return true ;
    }

    return false ;
  }

  /** clear the list, remove all entries */
  public void clear()
  {
    translations.clear();
  }

  public Enumeration elements()
  {
    return translations.elements() ;
  }

  /** sorts all entries - now only the key sorting is implemented
   *  and the langID is ignored */
  public void sortElements(int direction, int langID)
  {
    Object eList[] = translations.toArray() ;
    java.util.Arrays.sort(eList, new TKeyComparator(direction) );

      translations.clear();
      for (int t = 0 ; t < eList.length ; t++)
      {
        translations.add(eList[t]) ;
      }
  }


  /** returns the size */
  public int size()
  {
    return translations.size() ;
  }


  // -------------------------------------------------------------------------
  class TKeyComparator implements java.util.Comparator
  {
    private int sortDirection ;

    public TKeyComparator(int direction)
    {
      sortDirection = direction ;
    }
    public final int compare(Object o1, Object o2)
    {
      TMultiLanguageEntry p1 = null ;
      TMultiLanguageEntry p2 = null ;

      try
      {
        p1 = (TMultiLanguageEntry) o1 ;
        p2 = (TMultiLanguageEntry) o2 ;
      }
      catch (Exception e) {}

      int back = 0 ;

      if ((p1 != null) && (p2 != null))
      {
        switch (sortDirection)
        {
          case TLangProps.SORT_DOWNWARD :
            back = p1.getKey().compareToIgnoreCase( p2.getKey()) ;
            break ;
          case TLangProps.SORT_UPWARD :
            back = p1.getKey().compareToIgnoreCase( p2.getKey()) ;
            back = back * -1 ;
            break ;
          case TLangProps.SORT_TRANSLATIONSCOUNT_UPWARD :
            back = p1.getEmptyTranslationCount() - p2.getEmptyTranslationCount() ;
            break ;
          case TLangProps.SORT_TRANSLATIONSCOUNT_DOWNWARD :
            back = p2.getEmptyTranslationCount() - p1.getEmptyTranslationCount() ;
            break ;
          default :
            back = 0 ;
        }
      }

      return back ;
    }

    public final boolean equals(Object obj)
    {
      return false ;
    }

  }



}
