/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// created by : r.nagel 4.4.2005
//
// function : all graphical constants and look&feel routines
//
// todo     : theme handling
//
// modified :


package net.sf.langproper.gui ;

import java.net.*;
import javax.swing.ImageIcon;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.JFrame;
import javax.swing.plaf.metal.MetalTheme;
import javax.swing.UIManager;
import net.sf.langproper.engine.TLangProps;
import net.sf.langproper.gui.lang.LangManagingDialog;

public class GUIGlobals
{
  private static String pre = "/resource/images/" ;

  public static URL
      // load/save
      openIconName   = GUIGlobals.class.getResource(pre +"open.png"),
      saveIconName   = GUIGlobals.class.getResource(pre +"save.png"),

      // source/utf view
      viewSourceIconName   = GUIGlobals.class.getResource(pre +"txt.png"),
      viewIconName   = GUIGlobals.class.getResource(pre +"document.png"),

      // mark/unmark incomplete entries
      viewIncompleteIconName = GUIGlobals.class.getResource(pre +"view_detailed.png"),
      viewCompleteIconName  = GUIGlobals.class.getResource(pre +"view_text.png"),

      // TLangTextField status
      requireIconName   = GUIGlobals.class.getResource(pre +"require.png"),
      warningIconName   = GUIGlobals.class.getResource(pre +"achtung.png"),
      errorIconName   = GUIGlobals.class.getResource(pre +"problem.png"),

      // Table sort direction
      tableSortUpIconName   = GUIGlobals.class.getResource(pre +"1uparrow.png"),
      tableSortDownIconName   = GUIGlobals.class.getResource(pre +"1downarrow.png"),

      // comments available
      tableEntryCommentIconName   = GUIGlobals.class.getResource(pre +"attach.png")
      ;

  public static ImageIcon viewIcon2 ;
  public static ImageIcon viewIcon1 ;
  public static ImageIcon markIcon2 ;
  public static ImageIcon markIcon1 ;
  public static ImageIcon tableSortUpIcon ;
  public static ImageIcon tableSortDownIcon ;
  public static ImageIcon tableEntryCommentIcon ;

  // Application (popeye) mainframe
  public static JFrame APPLICATION_MAINFRAME = null ;

  // project dialog
  public static LangManagingDialog PROJECT_PROPERTIES_DIALOG = null ;

  //
  public static TOverviewPanel oPanel = null ;

 /** load icons and initialize other resources */
 public static void init()
 {
   try
   {
     viewIcon2 = new ImageIcon( GUIGlobals.viewIconName ) ;
     viewIcon1 = new ImageIcon( GUIGlobals.viewSourceIconName ) ;
     markIcon2 = new ImageIcon( GUIGlobals.viewIncompleteIconName ) ;
     markIcon1 = new ImageIcon( GUIGlobals.viewCompleteIconName ) ;
     tableSortUpIcon = new ImageIcon( tableSortUpIconName ) ;
     tableSortDownIcon = new ImageIcon( tableSortDownIconName ) ;
     tableEntryCommentIcon = new ImageIcon( tableEntryCommentIconName ) ;

   }
   catch (Exception e)
   {
     System.err.println( " Could not find icons " ) ;
   }
 }


 public static void updateTheme(MetalTheme aTheme, JFrame aFrame)
 {
    MetalLookAndFeel.setCurrentTheme( aTheme );
    try
    {
      UIManager.setLookAndFeel( new MetalLookAndFeel() );
    }
    catch ( UnsupportedLookAndFeelException ex )
    {
      System.out.println("Cannot set new Theme for Java Look and Feel.");
    }
    SwingUtilities.updateComponentTreeUI( aFrame );
 }


 /** returns a readable string of the sorting direction <id> */
 public static String getSortingDescription( int id )
 {
   String back = null ;
   switch ( id )
   {
     case TLangProps.SORT_DOWNWARD:
       back = "keys downward" ;
       break ;
     case TLangProps.SORT_UPWARD:
       back = "keys upward" ;
       break ;
     case TLangProps.SORT_TRANSLATIONSCOUNT_UPWARD:
       back = "number of translations upwards" ;
       break ;
     case TLangProps.SORT_TRANSLATIONSCOUNT_DOWNWARD:
       back = "number of translations downwards" ;
       break ;
     case TLangProps.SORT_NOTHING:
     default:
       back = "no sorting" ;
       break ;
   }
   return back ;
 }


}
