/*
 Popeye - Java (Language) Properties File Editor

 Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
 * Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


// created by : r.nagel 4.4.2005
//
// function : Application frame of "standalone" version
//
// todo     :
//
//
// modified :

package net.sf.langproper.gui ;

import java.awt.* ;
import javax.swing.* ;
import java.awt.event.ActionEvent ;
import java.io.* ;
import net.sf.langproper.* ;
import java.awt.event.WindowEvent ;
import java.awt.event.WindowAdapter ;
import net.sf.langproper.gui.actions.* ;

public class TMainFrame
    extends JFrame
{
  BorderLayout borderLayout1 = new BorderLayout() ;
  TOverviewPanel oPanel = new TOverviewPanel() ;
  JMenuBar jMenuBar1 = new JMenuBar() ;
  JMenu jMenu1 = new JMenu() ;
  JMenuItem loadMenuItem = new JMenuItem() ;
  JMenuItem saveMenuItem = new JMenuItem() ;
  JMenuItem exitMenuItem = new JMenuItem() ;
  JMenuItem newMenuItem = new JMenuItem() ;
  JMenu jMenu2 = new JMenu() ;
  JCheckBoxMenuItem showSourceMenuItem = new JCheckBoxMenuItem() ;
  JCheckBoxMenuItem incompleteMenuItem = new JCheckBoxMenuItem() ;
  JToolBar toolBar = new JToolBar() ;
  JToggleButton viewButton = new JToggleButton() ;
  JToggleButton markButton = new JToggleButton() ;

  TLoadAction loadAction = new TLoadAction() ;
  TSaveAction saveAction = new TSaveAction() ;
  TChangeDataViewAction changeViewAction = new TChangeDataViewAction() ;
  TMarkIncompleteViewAction markViewAction = new TMarkIncompleteViewAction() ;
  TNewEntryAction newEntryAction = new TNewEntryAction() ;
  TExitAction exitAction = new TExitAction() ;
  TProjectPropertiesAction langSelAction = new TProjectPropertiesAction( );
  JMenuItem selLangMenuItem = new JMenuItem( langSelAction ) ;
  JMenu jMenu3 = new JMenu() ;
  TOption_RB_Action replaceOptionAction = new  TOption_RB_Action() ;
  JCheckBoxMenuItem blanksMenuItem = new JCheckBoxMenuItem() ;

  public TMainFrame()
  {
    super() ;
    try
    {
      jbInit() ;
      this.setSize( 300, 500 ) ;
    }
    catch ( Exception exception )
    {
      System.out.println( "error creating main window" ) ;
      exception.printStackTrace() ;
    }
  }

  /* JBuilder GUI Designer Style */
  private void jbInit() throws Exception
  {
    this.setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE ) ;
    this.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent e )
      {
        exitRoutine() ;
      }
    } ) ;

    getContentPane().setLayout( borderLayout1 ) ;
    this.setJMenuBar( jMenuBar1 ) ;
    jMenu1.setText( "File" ) ;
    loadMenuItem.setText( "Load" ) ;
    jMenu2.setText( "View" ) ;
    showSourceMenuItem.setText( "source view" ) ;
    showSourceMenuItem.addActionListener( changeViewAction ) ;

    incompleteMenuItem.setText( "mark incomplete entries" ) ;
    incompleteMenuItem.addActionListener( markViewAction ) ;
    saveMenuItem.setText( "Save" ) ;
    exitMenuItem.setText( "Exit" ) ;
    newMenuItem.setText( "New Entry" ) ;
    jMenu3.setText( "Options" ) ;
    blanksMenuItem.setText( "replace blanks" ) ;

    //    selLangMenuItem.setText( "Language" ) ;
//    selLangMenuItem.addActionListener( langSelAction );

    this.getContentPane().add( oPanel, java.awt.BorderLayout.CENTER ) ;
    jMenuBar1.add( jMenu1 ) ;
    jMenuBar1.add( jMenu2 ) ;
    jMenuBar1.add( jMenu3 ) ;

    jMenu1.add( newMenuItem ) ;
    newMenuItem.addActionListener( newEntryAction ) ;
    jMenu1.addSeparator() ;
    jMenu1.add( loadMenuItem ) ;
    loadMenuItem.addActionListener( loadAction ) ;
    jMenu1.add( saveMenuItem ) ;
    saveMenuItem.addActionListener( this.saveAction ) ;
    jMenu1.addSeparator() ;
    jMenu1.add( selLangMenuItem ) ;
    jMenu1.addSeparator() ;
    jMenu1.add( exitMenuItem ) ;
    exitMenuItem.addActionListener( exitAction ) ;
    jMenu2.add( showSourceMenuItem ) ;
    jMenu2.add( incompleteMenuItem ) ;

    viewButton.setAction( changeViewAction ) ;
    markButton.setAction( markViewAction ) ;

    toolBar.add( loadAction ) ;
    toolBar.add( saveAction ) ;
    toolBar.addSeparator() ;
    toolBar.add( viewButton ) ;
    toolBar.add( markButton ) ;

    this.getContentPane().add( toolBar, BorderLayout.PAGE_START ) ;
    jMenu3.add( blanksMenuItem ) ;
    blanksMenuItem.setSelected( TGlobal.runtime.replaceSpaceInInput() );
    blanksMenuItem.addActionListener( replaceOptionAction ) ;


    //    saveMenuItem.setEnabled( false );
    saveAction.setEnabled( false ) ;
    newMenuItem.setEnabled( false ) ;

    this.setTitle( TGlobal.APPLICATION_NAME + " " + TGlobal.VERSION_STRING ) ;

    GUIGlobals.oPanel = oPanel ;

  }

  private void exitRoutine()
  {
    if ( TGlobal.current.isDataChanged() )
    {
      int result = JOptionPane.showConfirmDialog( null,
                                                  "Data changed - exit without saving files ?",
                                                  "Exit Question/Warning",
                                                  JOptionPane.YES_NO_OPTION ) ;

      if ( result == JOptionPane.YES_OPTION )
      {
        System.exit( 0 ) ;
      }
    }
    else
    {
      System.exit( 0 ) ;
    }
  }

  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------
  private class TLoadAction
      extends AbstractAction
  {
    private JFileChooser openDialog ;

    public TLoadAction()
    {
//      super("Load") ;
      super( "load", new ImageIcon( GUIGlobals.openIconName ) ) ;

      openDialog = new JFileChooser( TGlobal.DEFAULT_LOAD_PATH ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      int result ;
      if ( TGlobal.current.isDataChanged() )

      {
        result = JOptionPane.showConfirmDialog( null,
                                                "Data changed - load new data ?",
                                                "load Question/Warning",
                                                JOptionPane.YES_NO_OPTION ) ;

      }
      else
      {
        result = JOptionPane.YES_OPTION ;
      }

      if ( result == JOptionPane.YES_OPTION )
      {
        int returnVal = openDialog.showOpenDialog( null ) ;
        if ( returnVal == JFileChooser.APPROVE_OPTION )
        {
          File file = openDialog.getSelectedFile() ;
          TGlobal.current.load( file ) ;
          oPanel.updateStructure() ;
//        saveMenuItem.setEnabled( true );
          saveAction.setEnabled( true ) ;
          newMenuItem.setEnabled( true ) ;
           // delete old project properties dialog
          GUIGlobals.PROJECT_PROPERTIES_DIALOG = null ;
        }
      }

    }
  }

  // --------------------------------------------------------------------------
  private class TSaveAction
      extends AbstractAction
  {
    public TSaveAction()
    {
      super( "save", new ImageIcon( GUIGlobals.saveIconName ) ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      TGlobal.current.save() ;
    }
  }

  // --------------------------------------------------------------------------
  private class TChangeDataViewAction
      extends AbstractAction
  {
    public TChangeDataViewAction()
    {
//      super("view") ;
      super( "", GUIGlobals.viewIcon2 ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      boolean state ;
      if ( e.getSource() == showSourceMenuItem )
      {
        state = showSourceMenuItem.isSelected() ;
        viewButton.setSelected( state ) ;
      }
      else
      {
        state = viewButton.isSelected() ;
        showSourceMenuItem.setSelected( state ) ;
      }

      if ( state )
      {
        viewButton.setIcon( GUIGlobals.viewIcon1 ) ;
      }
      else
      {
        viewButton.setIcon( GUIGlobals.viewIcon2 ) ;
      }
      oPanel.setShowSource( state ) ;
    }
  }

  // --------------------------------------------------------------------------
  private class TMarkIncompleteViewAction
      extends AbstractAction
  {
    public TMarkIncompleteViewAction()
    {
      super( "", GUIGlobals.markIcon1 ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      boolean state ;
      if ( e.getSource() == incompleteMenuItem )
      {
        state = incompleteMenuItem.isSelected() ;
        markButton.setSelected( state ) ;
      }
      else
      {
        state = markButton.isSelected() ;
        incompleteMenuItem.setSelected( state ) ;
      }

      if ( state )
      {
        markButton.setIcon( GUIGlobals.markIcon2 ) ;
      }
      else
      {
        markButton.setIcon( GUIGlobals.markIcon1 ) ;
      }
      oPanel.setMarkIncomplete( state ) ;
    }
  }

  // --------------------------------------------------------------------------
  private class TExitAction
      extends AbstractAction
  {
    public TExitAction()
    {
      super( "Exit", GUIGlobals.viewIcon1 ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      exitRoutine() ;
    }
  }

  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  private class TOption_RB_Action  // Replace Blanks Action
      extends AbstractAction
  {
    public TOption_RB_Action()
    {
      super( "" ) ;

    }

    public void actionPerformed( ActionEvent e )
    {
      boolean state = !blanksMenuItem.isSelected() ;
      blanksMenuItem.setSelected( state) ;
      TGlobal.runtime.setReplaceSpaceInput(state);
    }
  }


}
