/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// created by : r.nagel 4.4.2005
//
// function : contains the JTable (all available entries) and the
//            JTextFieldPanel (all translations for one entry)
//
// todo     :
//
//
// modified :


package net.sf.langproper.gui ;

import java.awt.* ;
import javax.swing.* ;
import javax.swing.table.*;
import net.sf.langproper.TGlobal;
import javax.swing.event.*;
import net.sf.langproper.engine.*;
import java.awt.event.ComponentEvent;
import java.awt.event.*;
import net.sf.langproper.gui.actions.*;
import net.sf.langproper.gui.listener.*;
import net.sf.langproper.gui.fields.*;

public class TOverviewPanel
    extends JPanel
    implements ComponentListener, SortingListener, TableModelListener
{
  private BorderLayout borderLayout1 = new BorderLayout() ;
  private TPropTable propTable = new TPropTable();
  private TTextFieldPanel textfields = new TTextFieldPanel();
  private JSplitPane splitter = new JSplitPane( JSplitPane.VERTICAL_SPLIT) ;
  private JScrollPane scroller ;
  private JPopupMenu tableMenu = new JPopupMenu() ;

  private TStatusLine statusLine = new TStatusLine();


  /** show the source of the entries, e.g. "Y"="\\u00xx" => (real text)=(source) */
  private boolean showSource ;

  public TOverviewPanel()
  {
    this.setLayout( borderLayout1 ) ;

    this.addComponentListener( this );

    propTable.addListSelectionListener( textfields);
    propTable.addSortingListener( this );
    TGlobal.current.addTableModelListener(this);

    scroller = new JScrollPane(textfields) ;
    scroller.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
//    scroller.setMaximumSize( new Dimension(500, 500) );

//    this.add( scroller, java.awt.BorderLayout.SOUTH ) ;

    splitter.add( propTable, JSplitPane.TOP) ;
    splitter.add( scroller, JSplitPane.BOTTOM) ;
    splitter.setOneTouchExpandable(true) ;

    statusLine.setSortingDirectionLabel( TLangProps.SORT_NOTHING);

    this.add( splitter, BorderLayout.CENTER) ;

    this.add( statusLine, BorderLayout.SOUTH) ;

    TNewEntryAction newAction = TNewEntryAction.getInstance() ;
    newAction.setTable( propTable );
    tableMenu.add( newAction ) ;

    TRemoveEntryAction delAction = TRemoveEntryAction.getInstance() ;
    delAction.setTable( propTable );
    tableMenu.add( delAction ) ;

    propTable.setPopupMenu( tableMenu );
  }

  /** update the table view, some columns could be changed */
  public void updateStructure()
  {
    propTable.init();
    textfields.updateStructure();
    updateEntryCounter();
    resizeIT() ;
  }


  /** show entry source or real text */
  public boolean isShowSource()
  {
    return showSource ;
  }

  /** show entry source or real text */
  public void setShowSource( boolean pShowSource )
  {
    this.showSource = pShowSource ;
    propTable.setShowSource( showSource);
    textfields.setShowSource( showSource);
  }

  /** inclompete table entries should be marked */
  public boolean isMarkIncomplete()
  {
    return propTable.isMarkIncomplete();
  }

  /** inclompete table entries should be marked */
  public void setMarkIncomplete(boolean pMarkIncomplete)
  {
    propTable.setMarkIncomplete( pMarkIncomplete );
  }

  private void updateEntryCounter()
  {
    statusLine.setNumberOfEntries( TGlobal.current.getRowCount() );
  }

  private void resizeIT()
  {
    Dimension minSize = textfields.getRealSize() ;
    Dimension thSize = splitter.getSize() ;

    int dummy = minSize.height + scroller.getInsets().bottom +
                 scroller.getInsets().top ;

    int loc = splitter.getSize().height - splitter.getInsets().bottom
        - splitter.getDividerSize() - scroller.getInsets().bottom
        - dummy ;

    // textfields panel to small
    if (dummy > scroller.getSize().getHeight())
    {
      // only 50% of heigth allowed
      if (dummy < thSize.getHeight()/2)
      {
        splitter.setDividerLocation( loc ) ;
      }
    }
    else
    {
      splitter.setDividerLocation( loc ) ;
    }
  }

  // --------------------------------------------------------------------------
  // ComponentListener
  public void componentHidden( ComponentEvent e )
  {
  }

  public void componentMoved( ComponentEvent e )
  {
  }

  public void componentResized( ComponentEvent e )
  {
    resizeIT() ;
  }

  public void componentShown( ComponentEvent e )
  {
  }

  // --------------------------------------------------------------------------
  // SortingListener
  public void resortPerformed( int sortDirection )
  {
    statusLine.setSortingDirectionLabel( sortDirection );
  }

  // --------------------------------------------------------------------------
  // TableModelListener
  public void tableChanged( TableModelEvent e )
  {
    if ((e.getType() == TableModelEvent.DELETE) ||
        (e.getType() == TableModelEvent.INSERT))
    {
      this.updateEntryCounter();
    }
  }

}
