/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// created by : r.nagel 04.04.2005, based on TableDemo from java.sun.com
//
// function : table with translations
//
// todo     :
//
// modified :

package net.sf.langproper.gui ;

import javax.swing.JPanel ;
import javax.swing.JScrollPane ;
import javax.swing.JTable ;
import java.awt.Dimension ;
import java.awt.GridLayout ;
import net.sf.langproper.* ;
import javax.swing.* ;
import javax.swing.table.* ;
import java.awt.* ;
import net.sf.langproper.engine.* ;
import javax.swing.event.* ;
import java.awt.event.* ;
import net.sf.langproper.gui.listener.*;

public class TPropTable
    extends JPanel implements FocusListener
{
  private AbstractTableModel myModel ;
  private TPropTableRenderer renderer ;
  private boolean showSource ;
  private boolean markIncomplete ;
  private JTable table ;
  private JScrollPane scrollPane ;
  private JPopupMenu popupMenu = null ;

  private TMultiLanguageEntry savedSelectionEntry = null ;

  public TPropTable()
  {
    super( new GridLayout( 1, 0 ) ) ;

    showSource = false ;

    myModel = TGlobal.current ;

    // renderer instance
    renderer = new TPropTableRenderer() ;

    // create table and renderer
    table = new JTable( myModel )
    {
      public TableCellRenderer getCellRenderer( int row, int column )
      {
//        if ( ( row == 0 ) && ( column == 0 ) )
        {
          return renderer ;
        }
//        return super.getCellRenderer( row, column ) ;
      }

      //Implement table cell tool tips.
       public String getToolTipText(MouseEvent e) {
           String tip = null;
           java.awt.Point p = e.getPoint();
           int rowIndex = rowAtPoint(p);
           int colIndex = columnAtPoint(p);
           int realColumnIndex = convertColumnIndexToModel(colIndex);

           if (realColumnIndex == 0)   // comments
           {
               if (TGlobal.current.hasComment( rowIndex ) )
               {
                 tip = "comment available" ;
               }
               else tip = null ;
           } else {
               //You can omit this part if you know you don't
               //have any renderers that supply their own tool
               //tips.
               tip = super.getToolTipText(e);
           }
           return tip;
       }

    } ;

    table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION ) ;

//    table.getSelectionModel().addListSelectionListener( new TPropTableListSelection() );

    table.setPreferredScrollableViewportSize( new Dimension( 500, 70 ) ) ;

    table.setRowSelectionAllowed( true ) ;

    table.addFocusListener( this ) ;

    table.addMouseListener( new PopupListener() );

     init() ;
    /*
        // After fireTableStructureChanged() (e.g. load) is processed
        // a new ColumnModel is created
        //  !!! works only the first time - feasibility demo
        ButtonHeaderRenderer headerRenderer = new ButtonHeaderRenderer() ;
        TableColumn col = table.getColumnModel().getColumn( 0 ) ;
        col.setHeaderRenderer( headerRenderer ) ;
     */
    // column sorting
    JTableHeader header = table.getTableHeader() ;
    header.addMouseListener( new HeaderListener( header ) ) ;
    header.setReorderingAllowed( false );  // no dragging of columns to another place

    //Create the scroll pane and add the table to it.
    scrollPane = new JScrollPane( table ) ;

    //Add the scroll pane to this panel.
    add( scrollPane ) ;
  }

  public AbstractTableModel getModel()
  {
    return myModel ;
  }

  public boolean getShowSource()
  {
    return showSource ;
  }

  public void setShowSource( boolean pShowSource )
  {
    this.showSource = pShowSource ;
    myModel.fireTableRowsUpdated( 0, myModel.getRowCount() - 1 ) ;
  }

  public boolean isMarkIncomplete()
  {
    return markIncomplete ;
  }

  public void setMarkIncomplete( boolean pMarkIncomplete )
  {
    this.markIncomplete = pMarkIncomplete ;
//    myModel.fireTableDataChanged() ;
    myModel.fireTableRowsUpdated( 0, myModel.getRowCount() - 1 ) ;
  }

  public void addListSelectionListener( ListSelectionListener listener )
  {
    //Ask to be notified of selection changes.
    ListSelectionModel rowSM = table.getSelectionModel() ;
    rowSM.addListSelectionListener( listener ) ;
  }

  public void focusGained( FocusEvent e )
  {
//    System.out.println( "get focus" ) ;
  }

  public void focusLost( FocusEvent e )
  {
//    System.out.println( "lost focus" ) ;
  }

  private void saveSelection()
  {
    ListSelectionModel lsm = table.getSelectionModel() ;
    if ( lsm.isSelectionEmpty() )
    {
      //no rows are selected
      savedSelectionEntry = null ;
    }
    else
    {
      int selectedRow = lsm.getMinSelectionIndex() ;
      savedSelectionEntry = (TMultiLanguageEntry) myModel.getValueAt(selectedRow, 0) ;
    }
  }

  private void restoreSelection(boolean makeSelectionVisible)
  {
    if (savedSelectionEntry != null)
    {
      ListSelectionModel lsm = table.getSelectionModel() ;
      int index = lsm.getMinSelectionIndex() ;

      int old =TGlobal.current.getLanguageEntryIndex( savedSelectionEntry.getKey() ) ;

      if ( (index != old) && (old > -1))
      {
        lsm.setSelectionInterval( old, old ) ;
        ensureVisibility( old ) ;
      }
    }
  }

  /** select a table row */
  public void selectRow(int row, boolean makeVisible)
  {
    if ( (row > -1) && (row < myModel.getRowCount() ))
    {
      ListSelectionModel lsm = table.getSelectionModel() ;
      lsm.setSelectionInterval( row, row ) ;
      if (makeVisible)
      {
        ensureVisibility( row ) ;
      }
    }
  }

  /** make a table row visible  - it's a little bit buggy :-) */
  private void ensureVisibility(int row)
  {
    Rectangle rec = table.getCellRect(row, 0, true) ;
    Point p = rec.getLocation() ;

    JViewport view = scrollPane.getViewport() ;
    view.setViewPosition(p);
  }

  public int getSelectedIndex()
  {
    int back = -1 ;
    ListSelectionModel lsm = table.getSelectionModel() ;
    if ( ! lsm.isSelectionEmpty() )
    {
      back = lsm.getMinSelectionIndex() ;
    }

    return back ;
  }

  public void setPopupMenu( JPopupMenu menu )
  {
     popupMenu = menu ;
  }

  public void init()
  {
    TableColumn column = table.getColumnModel().getColumn(0) ;
    column.setMaxWidth(20);
    column.setMinWidth(16);
    column.setWidth(16);
    column.setResizable(false);
  }

  // --------------------------------------------------------------------------
  // handling of table column sorting
  // taken from Java 2 Platform Std.Ed.v1.4.2 API - EventListenerList

  public void addSortingListener( SortingListener listener )
  {
    if ( listenerList != null )
    {
      listenerList.add( SortingListener.class, listener ) ;
    }
  }

  public void removeActionListener( ActionListener listener )
  {
    if ( listenerList != null )
    {
      listenerList.remove( ActionListener.class, listener ) ;
    }
  }

  protected void fireReSortPerformed( int id )
  {
    // Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList() ;

    // Process the listeners last to first, notifying
    // those that are interested in this event
    for ( int i = listeners.length -2 ; i >= 0 ; i-=2 )
    {
      if ( listeners[i] == SortingListener.class )
      {
        ( ( SortingListener ) listeners[i+1] ).resortPerformed( id ) ;
      }
    }
  }

  // --------------------------------------------------------------------------
  // Table Renderer
  // handling of even/odd lines (colors) and printing the right content
  // --------------------------------------------------------------------------

  public class TPropTableRenderer
      extends JLabel implements TableCellRenderer
  {

    private Color selectedColor = new Color( 26, 70, 159 ) ; // 0x1A469F
    private Color oddColor = new Color( 0xdfdfdf ) ;

    private Color oddMarkedColor = new Color( 0xf1c7c0 ) ;
    private Color evenMarkedColor = new Color( 0xffe1e1 ) ;

    public TPropTableRenderer()
    {
      setOpaque( true ) ; //MUST do this for background to show up.
      this.setFont( new Font( "Arial", Font.PLAIN, 12 ) ) ;
    }

    public Component getTableCellRendererComponent(
        JTable table, Object value,
        boolean isSelected, boolean hasFocus,
        int row, int column )
    {
      boolean marked = false ;

      if ( value != null )
      {
        TMultiLanguageEntry entry = ( TMultiLanguageEntry ) value ;

        String str = TGlobal.current.getTranslation( entry, column-1) ;  // first column = status

        // set text
        if ( showSource )
        {
          this.setText( str ) ;
        }
        else
        {
          this.setText( Utils.get_WYSIWYG_String( str ) ) ;
        }

        // incomplete entry ?
        // every column call's this method -> for loop (optimize it ?)
        marked = !entry.isComplete() ;

        // has this entry any comment -> paint comments icon in status column
        if ((column == 0) && ( entry.hasComment() ))
        {
          this.setIcon( GUIGlobals.tableEntryCommentIcon ) ;
        }
        else this.setIcon( null ) ;
      }
      else // value empty
      {
        this.setText( "" ) ;
      }

      Color background ;

      if ( !isSelected )
      {
        // mark incomplete lines
        marked = marked & isMarkIncomplete() ;

        if ( ( row % 2 ) == 1 ) // odd line
        {
          if ( marked )
          {
            background = oddMarkedColor ;
          }
          else
          {
            background = oddColor ;
          }
        }
        else // even line
        {
          if ( marked )
          {
            background = evenMarkedColor ;
          }
          else
          {
            background = Color.white ;
          }
        }
        this.setForeground( Color.black ) ;
      }
      else // selected line
      {
        background = selectedColor ;
        this.setForeground( Color.lightGray ) ;
      }
      this.setBackground( background ) ;

      return this ;
    }
  }

  // ---------------------------------------------------------------------------
  // Header handling

  // HeaderRenderer - not used !!!!
  class ButtonHeaderRenderer
      extends JLabel implements TableCellRenderer
  {
    public Component getTableCellRendererComponent( JTable table, Object value,
        boolean isSelected, boolean hasFocus, int row, int column )
    {
      int sort = TGlobal.current.getSortDirection() ;

      this.setText( TGlobal.current.getColumnName( column ) ) ;

      switch ( sort )
      {
        case TLangProps.SORT_DOWNWARD:
          this.setIcon( GUIGlobals.tableSortDownIcon ) ;
          break ;
        case TLangProps.SORT_UPWARD:
          this.setIcon( GUIGlobals.tableSortUpIcon ) ;
          break ;
        default:
          this.setIcon( null ) ;
      }
      return this ;
    }

  }

  // header listener - sorting
  public class HeaderListener
      extends MouseAdapter
  {
    private JTableHeader header ;

    HeaderListener( JTableHeader header )
    {
      this.header = header ;
    }

    public void mousePressed( MouseEvent e )
    {
      /*
       * If the user adjusts the header size the sort event is
       * always triggered.
       * To avoid this behaviour we check if the mouse is
       * inside the label's bounds and has a certain distance (offset)
       * to the label border.
       *
       * Sascha Hunold <hunoldinho@users.sourceforge.net>
       */

      Point p = e.getPoint() ;
      int colindex = header.columnAtPoint( p ) ;
      if ( colindex >= 0 )
      {
        final int initoffset = 3 ;
        int xoffset = initoffset ;
        for ( int i = 0 ; i < colindex ; i++ )
        {
          xoffset += table.getColumnModel().getColumn( i ).getWidth() ;
        }
        TableColumn column = table.getColumnModel().getColumn( colindex ) ;
        int cw = column.getWidth() ;
        int ch = header.getHeight() ;

        Rectangle r = new Rectangle() ;

        r.setBounds( xoffset, 0 /*offset*/, cw - 2 * initoffset,
                     ch /*-2*offset*/ ) ;

        if ( !r.contains( p ) )
        {
          return ;
        }
      }

      if (colindex == 1)
      {
        int sort = TGlobal.current.getSortDirection() ;
        switch ( sort )
        {
          case TLangProps.SORT_DOWNWARD:
            sort = TLangProps.SORT_UPWARD ;
            break ;
          case TLangProps.SORT_UPWARD:
            sort = TLangProps.SORT_TRANSLATIONSCOUNT_UPWARD ;
            break ;
          case TLangProps.SORT_TRANSLATIONSCOUNT_UPWARD:
            sort = TLangProps.SORT_TRANSLATIONSCOUNT_DOWNWARD ;
            break ;
          case TLangProps.SORT_TRANSLATIONSCOUNT_DOWNWARD:
            sort = TLangProps.SORT_DOWNWARD ;
            break ;
          case TLangProps.SORT_NOTHING:
            sort = TLangProps.SORT_DOWNWARD ;
            break ;
          default:
            sort = TLangProps.SORT_NOTHING ;
            break ;
        }

        // save the current selction, because the sorting method will remove it
        saveSelection() ;
        TGlobal.current.setSortDirection( sort ) ;
        fireReSortPerformed( sort ) ;
        restoreSelection( true ) ;

        header.repaint() ;
      }
    }

    public void mouseReleased( MouseEvent e )
    {
      int col = header.columnAtPoint( e.getPoint() ) ;
      if ( col == 1 ) // key
      {
        header.repaint() ;
      }
    }
  }

  //---------------------------------------------------------------
  // table right click popup menu
  class PopupListener extends MouseAdapter
  {
    public void mousePressed( MouseEvent e )
    {
      maybeShowPopup( e ) ;
    }

    public void mouseReleased( MouseEvent e )
    {
      maybeShowPopup( e ) ;
    }

    private void maybeShowPopup( MouseEvent e )
    {
      if ( e.isPopupTrigger() )
      {
        popupMenu.show( e.getComponent(), e.getX(), e.getY() ) ;
      }
    }
  }



  // --------------------------------------------------------------------------
}
