/*
 Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
 * Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


// created by : r.nagel 4.4.2005
//
// function : JTextField with uniquie ID
//
// todo     :
//
// modified :


package net.sf.langproper.gui.fields ;

import javax.swing.* ;
import java.awt.event.* ;
import java.awt.Component ;

public class IDTextField
    extends JTextField

{
  private int myID ;

  private PopupListener listener ;

  private FieldPopup fieldPopupMenu ;

  /** data were changed - every data manipulation operation must set this flag */
  private boolean dataChanged = false ;

  public IDTextField( int ID )
  {
    myID = ID ;

    fieldPopupMenu = new FieldPopup() ;

    listener = new PopupListener( fieldPopupMenu ) ;

    this.addMouseListener( listener ) ;
  }

  public int getMyID()
  {
    return myID ;
  }

  public void setMyID( int pMyID )
  {
    this.myID = pMyID ;
  }

  public void setText( String txt )
  {
    super.setText( txt ) ;
    setDataChanged( false ) ;
  }

  public void cut()
  {
    // if something selected
    String str = this.getSelectedText() ;
    if (str != null)
      if (str.length() > 0)
        setDataChanged( true ) ;

    super.cut();
  }

  public void paste()
  {
    super.paste();
    setDataChanged( true ) ;
  }

  /** returns the status of data */
  public boolean isDataChanged()
  {
    return dataChanged ;
  }

  protected void setDataChanged( boolean value )
  {
    dataChanged = value ;
  }

  public FieldPopup getPopupMenu()
  {
    return fieldPopupMenu ;
  }

  // table right click popup menu
  class PopupListener
      extends MouseAdapter
  {
    private FieldPopup popMenu ;

    public PopupListener( FieldPopup menu )
    {
      popMenu = menu ;
    }

    public void mousePressed( MouseEvent e )
    {
      maybeShowPopup( e ) ;
    }

    public void mouseReleased( MouseEvent e )
    {
      maybeShowPopup( e ) ;
    }

    private void maybeShowPopup( MouseEvent e )
    {
      if ( e.isPopupTrigger() )
      {
        popMenu.setSenderID( myID ) ;
        popMenu.show( e.getComponent(), e.getX(), e.getY() ) ;
      }
    }
  }

}
