/*
 Popeye - Java (Language) Properties File Editor

 Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
 * Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


// created by : r.nagel 4.4.2005
//
// function : JTextField and Label
//
// todo     : - mark the field if some data changed
//            - mark the label (bold font?) if the textfield has the focus
//
// modified :


package net.sf.langproper.gui.fields ;

import javax.swing.* ;
import java.awt.event.* ;
import java.awt.* ;
import javax.swing.event.EventListenerList ;

import net.sf.langproper.gui.* ;

public class TLangTextField
    extends JPanel implements KeyListener, FocusListener
{

  public static final int
      NOTHING_STATE = 1,
      ERROR_STATE = 2,
      REQUIRE_STATE = 3,
      WARNING_STATE = 4,
      USER_STATE = 5 // user definied status
      ;

  private static final ImageIcon
      warningIcon = new ImageIcon( GUIGlobals.warningIconName ),
      errorIcon = new ImageIcon( GUIGlobals.errorIconName ),
      requireIcon = new ImageIcon( GUIGlobals.requireIconName ) ;

  private ImageIcon userIcon = null ; // user icon
  private int fieldState ;
  private JLabel imageLabel ;

  private JPanel bigPanel ;
  private IDTextField textField ;
  private JLabel label ;

  private SpringLayout layout ;

  private EventListenerList listenerList ;

  // auf diese Komponente wird der Focus nach einem Return/Enter gesetzt
  private JComponent nextFocusable ;

  /** creates a JTextfield with JLabel (contains labelText) and an identifier id */
  public TLangTextField( String labelText, int id )
  {
    super() ;

    this.setOpaque( false ) ;
    this.setLayout( new OverlayLayout( this ) ) ;
//    this.setBorder( BorderFactory.createEtchedBorder());

    listenerList = new EventListenerList() ;

    layout = new SpringLayout() ;
    bigPanel = new JPanel( layout ) ;
    bigPanel.setOpaque( false ) ;

    textField = new IDTextField( id ) ;
    textField.addKeyListener( this ) ;
    textField.addFocusListener( this ) ;
    label = new JLabel( labelText, JLabel.TRAILING ) ;
    label.setLabelFor( textField ) ;
    label.addFocusListener( this ) ;

    textField.setMaximumSize( new Dimension( 2000, 30 ) ) ;

    bigPanel.add( label ) ;
    bigPanel.add( textField ) ;

    //Adjust constraints for the label so it's at (5,5).
    layout.putConstraint( SpringLayout.WEST, label,
                          10,
                          SpringLayout.WEST, bigPanel ) ;
    layout.putConstraint( SpringLayout.NORTH, label,
                          6,
                          SpringLayout.NORTH, bigPanel ) ;

    //Adjust constraints for the text field so it's at
    //(<label's right edge> + 5, 5).
    layout.putConstraint( SpringLayout.WEST, textField,
                          10,
                          SpringLayout.EAST, label ) ;
    layout.putConstraint( SpringLayout.NORTH, textField,
                          3,
                          SpringLayout.NORTH, bigPanel ) ;

    //Adjust constraints for the content pane: Its right
    //edge should be 5 pixels beyond the text field's right
    //edge, and its bottom edge should be 5 pixels beyond
    //the bottom edge of the tallest component (which we'll
    //assume is textField).
    layout.putConstraint( SpringLayout.EAST, bigPanel,
                          5,
                          SpringLayout.EAST, textField ) ;
    layout.putConstraint( SpringLayout.SOUTH, bigPanel,
                          5,
                          SpringLayout.SOUTH, textField ) ;

    imageLabel = new JLabel() ;
    fieldState = REQUIRE_STATE ;
    setImage() ;

    this.add( imageLabel ) ;
    this.add( bigPanel ) ;

    // pipe all focus events to local class methods
    super.addFocusListener( this ) ;

    // textfield - popupMenu
    FieldPopup pop = textField.getPopupMenu() ;
    pop.setMenuTitle( labelText ) ;
    pop.add( new TCutAction() ) ;
    pop.add( new TCopyAction() ) ;
    pop.add( new TPasteAction() ) ;
  }

  // --------------------------------------------------------------------------
  public String getText()
  {
    return textField.getText() ;
  }

  public void setText( String newText )
  {
    textField.setText( newText ) ;
  }

  // --------------------------------------------------------------------------
  public void setLabelText( String labelText )
  {
    label.setText( labelText ) ;
  }

  public String getLabelText()
  {
    return label.getText() ;
  }

  // --------------------------------------------------------------------------
  public int getID()
  {
    return textField.getMyID() ;
  }

  public void setID( int pMyID )
  {
    textField.setMyID( pMyID ) ;
  }

  // --------------------------------------------------------------------------

  public void paint( Graphics g )
  {
    imageLabel.setLocation( textField.getX() - 5, 2 ) ;
    super.paint( g ) ;
  }

  // icon handling ------------------------------------------------------------
  // setzt anhand des Status (fieldState) das entsprechende Icon
  private final void setImage()
  {
    switch ( fieldState )
    {
      case ERROR_STATE:
        imageLabel.setIcon( errorIcon ) ;
        imageLabel.setToolTipText( "failure" ) ;
        break ;
      case REQUIRE_STATE:
        imageLabel.setIcon( requireIcon ) ;
        imageLabel.setToolTipText( "required" ) ;
        break ;
      case WARNING_STATE:
        imageLabel.setIcon( warningIcon ) ;
        imageLabel.setToolTipText( "warning" ) ;
        break ;
      case USER_STATE:
        imageLabel.setIcon( userIcon ) ;
        imageLabel.setToolTipText( "user message" ) ;
        break ;
      default: //         case NOTHING_STATE :
        imageLabel.setIcon( null ) ;
        imageLabel.setToolTipText( "" ) ;
    }
    repaint() ;
  }

  // user defined icon
  public void setUserIcon( ImageIcon icon )
  {
    userIcon = icon ;
  }

  public ImageIcon getUserIcon()
  {
    return userIcon ;
  }

  /** input status */
  public final void setStatus( int status )
  {
    fieldState = status ;
    setImage() ;
  }

  public final int getStatus()
  {
    return fieldState ;
  }

  /** returns the status of data */
  public boolean isDataChanged()
  {
    return textField.isDataChanged() ;
  }

  // --------------------------------------------------------------------------
  // Action handling
  // taken from Java 2 Platform Std.Ed.v1.4.2 API - EventListenerList

  public void addActionListener( ActionListener listener )
  {
    if ( listenerList != null )
    {
      listenerList.add( ActionListener.class, listener ) ;
    }
  }

  public void removeActionListener( ActionListener listener )
  {
    if ( listenerList != null )
    {
      listenerList.remove( ActionListener.class, listener ) ;
    }
  }

  protected void fireActionPerformed( String command )
  {
    ActionEvent foo = new ActionEvent( this, this.getID(), command ) ;

    // Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList() ;

    // Process the listeners last to first, notifying
    // those that are interested in this event
    for ( int i = listeners.length - 2 ; i >= 0 ; i -= 2 )
    {
      if ( listeners[i] == ActionListener.class )
      {
        ( ( ActionListener ) listeners[i + 1] ).actionPerformed( foo ) ;
      }
    }
  }

  // --------------------------------------------------------------------------
  // Key handling
  public void addKeyListener( KeyListener listener )
  {
    textField.addKeyListener( listener ) ;
  }

  // KeyListener interface ---------------
  public void keyPressed( KeyEvent e )
  {
  }

  public void keyReleased( KeyEvent e )
  {
  }

  public void keyTyped( KeyEvent e )
  {
    char code = e.getKeyChar() ;
    if ( code == e.VK_ENTER )
    {
//      if (dataField != null)
      {
        e.setKeyChar( e.CHAR_UNDEFINED ) ;
        e.setKeyCode( -1 ) ;

        // only fire this event, if the data has been changed
        if ( textField.isDataChanged() )
        {
          fireActionPerformed( "enter" ) ;
        }

        // not shift + enter => focus to next
        if ( ( e.getModifiers() & e.SHIFT_MASK ) != e.SHIFT_MASK )
        {
          this.changeFocusToNext() ;
        }
      }
    }
    else
    {
      textField.setDataChanged( true ) ;
    }
  }

  // --------------------------------------------------------------------------
  // focus handling

  // msg from JTextfield
  public void focusGained( FocusEvent e )
  {
    if ( e.getSource() == textField )
    {
      this.fireFocusGained( e.isTemporary() ) ;
    }
    else // if the label get's the focus then delegate it to textfield
    {
      textField.grabFocus() ;
    }
  }

  // msg from JTextfield
  public void focusLost( FocusEvent e )
  {
    if ( e.getSource() == textField )
    {
      this.fireFocusLost( e.isTemporary() ) ;
    }
  }

  public void addFocusListener( FocusListener listener )
  {
    if ( listenerList != null )
    {
      listenerList.add( FocusListener.class, listener ) ;
    }
  }

  public void removeFocusListener( FocusListener listener )
  {
    if ( listenerList != null )
    {
      listenerList.remove( FocusListener.class, listener ) ;
    }
  }

  protected void fireFocusLost( boolean temporary )
  {
    FocusEvent foo = new FocusEvent( this, this.getID(), temporary ) ;
    // Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList() ;
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for ( int i = listeners.length - 2 ; i >= 0 ; i -= 2 )
    {
      if ( listeners[i] == FocusListener.class )
      {
        ( ( FocusListener ) listeners[i + 1] ).focusLost( foo ) ;
      }
    }
  }

  protected void fireFocusGained( boolean temporary )
  {
    FocusEvent foo = new FocusEvent( this, this.getID(), temporary ) ;
    // Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList() ;
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for ( int i = listeners.length - 2 ; i >= 0 ; i -= 2 )
    {
      if ( listeners[i] == FocusListener.class )
      {
        ( ( FocusListener ) listeners[i + 1] ).focusGained( foo ) ;
      }
    }
  }

  public void setNextFocusable( JComponent nextfocus )
  {
    nextFocusable = nextfocus ;
  }

  public JComponent getNextFocusable()
  {
    return nextFocusable ;
  }

  public void changeFocusToNext()
  {
    if ( nextFocusable != null )
    {
      nextFocusable.grabFocus() ;
    }
  }

  public String toString()
  {
    return ( "TLangTextField <" + this.label.getText() + "><" + this.getID() +
             ">" ) ;
  }

  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // Menu Actions
  private class TCopyAction
      extends AbstractAction
  {
    public TCopyAction()
    {
      super( "copy" ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      textField.copy() ;
    }
  }

  // --------------------------------------------------------------------------
  private class TPasteAction
      extends AbstractAction
  {
    public TPasteAction()
    {
      super( "paste" ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      textField.paste() ;
    }
  }

  // --------------------------------------------------------------------------
  private class TCutAction
      extends AbstractAction
  {
    public TCutAction()
    {
      super( "cut" ) ;
    }

    public void actionPerformed( ActionEvent e )
    {
      textField.cut() ;
    }
  }

  // --------------------------------------------------------------------------
  // Example Action
  /*
    private class TExtendedAction extends AbstractAction
    {
      public TExtendedAction()
      {
        super( "example" ) ;
      }

      public void actionPerformed( ActionEvent e )
      {
        if (e.getSource() instanceof FieldPopup)
          {
            FieldPopup popup = (FieldPopup) e.getSource() ;
            System.out.println( "ID " + popup.getSenderID() ) ;
            System.out.println( "extended Action" +e.getID()) ;
          }
      }
    }
   */

  // --------------------------------------------------------------------------

  /*
     Quick & Dirty Example:

     mark textparts in jtextfield

     import java.awt.*;
     import javax.swing.*;
     import javax.swing.text.*;

     public class Test {
         public static final void main( String[] args ) {
             try {
                 JFrame f = new JFrame("Test");
                 f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                 JPanel p = new JPanel();
                 JTextField tf = new JTextField("TEST");
                 tf.getHighlighter().addHighlight( 0, 2,
                     new DefaultHighlighter.DefaultHighlightPainter(
                         Color.RED) );
                 p.add( tf );
                 f.getContentPane().add( p );
                 f.pack();
                 f.setVisible( true );
             }
             catch ( Exception e ) {
                 e.printStackTrace();
             }
         }
     }
   */
}
