/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// created by : r.nagel 4.4.2005
//
// function : contains all translations for one entry and allows to edit it
//
// todo     :
//
//
// modified :


package net.sf.langproper.gui.fields;

import javax.swing.* ;
import java.awt.* ;
import net.sf.langproper.* ;
import net.sf.langproper.engine.* ;
import java.awt.event.* ;
import javax.swing.event.ListSelectionListener ;
import javax.swing.event.ListSelectionEvent ;
import javax.swing.event.*;
import java.util.logging.* ;


public class TTextFieldPanel
    extends JPanel implements ListSelectionListener,
                              ActionListener,
                              FocusListener
{

  private boolean reportDataChangingOnFocusLost = true ;

  // input textfields
  private TLangTextField langFields[] ;

  private TMultiLanguageEntry currentEntry ;

  private boolean showSource ;

  //private SpringLayout layout ;
  private BoxLayout layout ;

  private int activeEntry = -1 ;

  public TTextFieldPanel()
  {
    super() ;
    try
    {
      jbInit() ;
    }
    catch ( Exception ex )
    {
      Logger.global.fine( "failure : create Textfields" ) ;
      ex.printStackTrace() ;
    }
  }

  private void jbInit() throws Exception
  {
//    layout = new SpringLayout() ;

    layout = new BoxLayout(this, BoxLayout.Y_AXIS) ;
    this.setLayout( layout ) ;
//    this.setBorder( BorderFactory.createEtchedBorder());
    generateFields() ;
  }

  /** the panel structure has changed */
  public void updateStructure()
  {
    try
    {
      clear() ;
      generateFields() ;
      layout.invalidateLayout(this);
      this.updateUI() ;
    }
    catch ( Exception ex )
    {
      System.out.println( "failure" ) ;
      ex.printStackTrace() ;
    }
  }

  /** removes all components and listeners */
  private void clear()
  {
    int len = langFields.length ;
    for (int t = 0 ; t < len ; t++)
    {
      TLangTextField field = langFields[t] ;
      field.removeActionListener( this );
      field.removeFocusListener( this);
    }

    this.removeAll();
  }

  /** put all textfield components into the panel */
  private void generateFields()
  {
    TLangProps data = TGlobal.current ;
    if ( data != null )
    {
      // get the number of language fields
      int len = data.getLangSize() ;

      // if default entry available counter +1
      if ( data.hasDefaultEntry() ) { len++ ; }

      // key field
      len++ ;

      // make the array
      langFields = new TLangTextField[len] ;

      // insert key field
      TLangTextField keyfield = new TLangTextField( "key", 0) ;
      keyfield.addActionListener(this);
      keyfield.addFocusListener( this );
      langFields[0] = keyfield ;
      keyfield.setMinimumSize( new Dimension( 200, 22 ) ) ;
      keyfield.setPreferredSize( new Dimension( 200, 30 ) ) ;
      this.add(keyfield) ;

      // generate all other language fields
      for ( int t = 1 ; t < len ; t++ )
      {
        TLangTextField field = new TLangTextField( data.getColumnName( t+1 ), t ) ;
        field.addActionListener(this);
        field.addFocusListener( this );
        langFields[t] = field ;
//        field.setMaximumSize( new Dimension( 1000, 22 ) ) ;
        field.setMinimumSize( new Dimension( 200, 22 ) ) ;
        field.setPreferredSize( new Dimension( 200, 30 ) ) ;

        langFields[t-1].setNextFocusable( field );

        this.add( field ) ;
      }

      langFields[len-1].setNextFocusable( keyfield );
    }
  }

  public void repaintData()
  {
    if ( currentEntry != null )
    {
      int len = langFields.length ;
      for ( int t = 0 ; t < len ; t++ )
      {
        String str = TGlobal.current.getTranslation( currentEntry, t ) ;
        TLangTextField field = langFields[t] ;

        if (str == null)
        {
          str = "" ;
          field.setStatus( TLangTextField.REQUIRE_STATE);
        }
        else if (str.length() < 1)
        {
          field.setStatus( TLangTextField.REQUIRE_STATE);
        }
        else  // something available
        {
          field.setStatus( TLangTextField.NOTHING_STATE);
        }

        // with/without unicode
        if ( showSource )
        {
          field.setText( str );
        }
        else field.setText( Utils.get_WYSIWYG_String( str));
      }
    }
    else // no entry data available - clear all fields
    {
      int len = langFields.length ;
      for ( int t = 0 ; t < len ; t++ )
      {
        TLangTextField field = langFields[t] ;
        field.setText("");
        field.setStatus( TLangTextField.REQUIRE_STATE);
      }

    }
  }

  public boolean isShowSource()
  {
    return showSource ;
  }

  public void setShowSource( boolean pShowSource )
  {
    this.showSource = pShowSource ;
    repaintData() ;
  }

  public Dimension getRealSize()
  {
//    return layout.minimumLayoutSize(this) ;
    return layout.preferredLayoutSize(this) ;
  }


  public void setData( TMultiLanguageEntry entry )
  {
    currentEntry = entry ;
    repaintData() ;
  }

  /** put the data from TLangTextField with id <id> into the translation set */
  private void adoptData(int id)
  {
    if (currentEntry != null)
    {
      TLangTextField field = langFields[id] ;
      String str = field.getText() ;
       // convert all whitespaces?
      if ( TGlobal.runtime.replaceSpaceInInput() )
        str = str.replaceAll(" ", "_") ;

      String valid = Utils.get_Unicode_I18n_String( str ) ;
      TGlobal.current.replaceTranslation( currentEntry, id, valid ) ;
      if ( showSource )
        field.setText( valid ) ;
      else
        field.setText( Utils.get_WYSIWYG_String( str) ) ;

      currentEntry.setStatus( TMultiLanguageEntry.STATE_MODIFIED ) ;
      field.setStatus( TLangTextField.NOTHING_STATE ) ;
    }
  }

  /** put the selected data from translation set into the TLangTextFields */
  private void fetchData()
  {
    // activeEntry is set by valueChanged()
    currentEntry = ( TMultiLanguageEntry ) TGlobal.current.getValueAt(
            activeEntry, 0 ) ; // column attribut has no effect
    this.setData( currentEntry ) ; // column attribut has no effect
  }

  // --------------------------------------------------------------------------
  // ActionListener
    public void actionPerformed( ActionEvent e )
    {
//      System.out.println("Action from " +e.getID() +" " +e.getActionCommand()) ;

      // enter comes only, if the data of sender has been changed
      if (e.getActionCommand().hashCode() == "enter".hashCode())
      {
         adoptData( e.getID() ) ;
      }
    }

  // --------------------------------------------------------------------------
  // ListSelectionListener - JTable
  public void valueChanged( ListSelectionEvent e )
  {

    ListSelectionModel lsm = ( ListSelectionModel ) e.getSource() ;
    activeEntry = lsm.getMinSelectionIndex() ;

      //Ignore extra messages.
      if ( e.getValueIsAdjusting() )
      {
        return ;
      }
    if ( lsm.isSelectionEmpty() )
    {
      //no rows are selected
    }
    else
    {
      fetchData() ;
    }
  }


  // --------------------------------------------------------------------------
  // focus handling of the TLangFields
  public void focusGained( FocusEvent e )
  {

  }

  public void focusLost( FocusEvent e )
  {
    int id = e.getID() ;
    TLangTextField field = langFields[id] ;
    if (field.isDataChanged())
    {
      if (this.reportDataChangingOnFocusLost)
      {
        if ( JOptionPane.showConfirmDialog( null,
                           "Data changed - apply ?",
                           "question",
                           JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION )
        {
          this.adoptData( id ) ;
        }
        // If the JOptionPane pops up, then the no fireValueChanged() from
        // JTable arrives. Update the data manually.
        this.fetchData();
      }
    }
  }

}
